"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
// importing this solves issues with __awaiter not defined using await in Android HeadlessTask
require("nativescript-tslib");
var background_geolocation_1 = require("./background-geolocation");
exports.BackgroundGeolocation = background_geolocation_1.BackgroundGeolocation;
require("./HeadlessJobService");
require("./HeadlessBroadcastReceiver");
var app = require("application");
var TSConfig = com.transistorsoft.locationmanager.adapter.TSConfig;
var TSCallback = com.transistorsoft.locationmanager.adapter.callback.TSCallback;
var TSLocationCallback = com.transistorsoft.locationmanager.adapter.callback.TSLocationCallback;
var TSLocation = com.transistorsoft.locationmanager.location.TSLocation;
var TSPlayServicesConnectErrorCallback = com.transistorsoft.locationmanager.adapter.callback.TSPlayServicesConnectErrorCallback;
var TSSyncCallback = com.transistorsoft.locationmanager.adapter.callback.TSSyncCallback;
var TSGetLocationsCallback = com.transistorsoft.locationmanager.adapter.callback.TSGetLocationsCallback;
var TSGetCountCallback = com.transistorsoft.locationmanager.adapter.callback.TSGetCountCallback;
var TSInsertLocationCallback = com.transistorsoft.locationmanager.adapter.callback.TSInsertLocationCallback;
var TSGetGeofencesCallback = com.transistorsoft.locationmanager.adapter.callback.TSGetGeofencesCallback;
var TSGetLogCallback = com.transistorsoft.locationmanager.adapter.callback.TSGetLogCallback;
var TSEmailLogCallback = com.transistorsoft.locationmanager.adapter.callback.TSEmailLogCallback;
var TSActivityChangeCallback = com.transistorsoft.locationmanager.adapter.callback.TSActivityChangeCallback;
var TSHttpResponseCallback = com.transistorsoft.locationmanager.adapter.callback.TSHttpResponseCallback;
var TSGeofenceCallback = com.transistorsoft.locationmanager.adapter.callback.TSGeofenceCallback;
var TSGeofencesChangeCallback = com.transistorsoft.locationmanager.adapter.callback.TSGeofencesChangeCallback;
var TSHeartbeatCallback = com.transistorsoft.locationmanager.adapter.callback.TSHeartbeatCallback;
var TSLocationProviderChangeCallback = com.transistorsoft.locationmanager.adapter.callback.TSLocationProviderChangeCallback;
var TSScheduleCallback = com.transistorsoft.locationmanager.adapter.callback.TSScheduleCallback;
var TSPowerSaveChangeCallback = com.transistorsoft.locationmanager.adapter.callback.TSPowerSaveChangeCallback;
var TSEnabledChangeCallback = com.transistorsoft.locationmanager.adapter.callback.TSEnabledChangeCallback;
var TSConnectivityChangeCallback = com.transistorsoft.locationmanager.adapter.callback.TSConnectivityChangeCallback;
var TSCurrentPositionRequest = com.transistorsoft.locationmanager.location.TSCurrentPositionRequest;
var TSWatchPositionRequest = com.transistorsoft.locationmanager.location.TSWatchPositionRequest;
var TSSensors = com.transistorsoft.locationmanager.util.Sensors;
var TSGeofence = com.transistorsoft.locationmanager.geofence.TSGeofence;
var TSLog = com.transistorsoft.locationmanager.logger.TSLog;
var TAG = "TSLocationnManager";
var REQUEST_ACTION_START = 1;
var REQUEST_ACTION_GET_CURRENT_POSITION = 2;
var REQUEST_ACTION_START_GEOFENCES = 3;
var emptyFn = function () { };
var HEADLESS_JOB_SERVICE = "com.transistorsoft.backgroundgeolocation.HeadlessJobService";
// Inform adapter.BackgroundGeolocation when Activity is destroyed.
app.android.on(app.AndroidApplication.activityDestroyedEvent, function (args) {
    console.log('[BackgroundGeolocation] onActivityDestroyedEvent -', args.activity);
    if (!app.android.startActivity) {
        Api.onActivityDestroyed(args);
    }
});
var Api = /** @class */ (function () {
    function Api() {
    }
    Api.onActivityDestroyed = function (args) {
        com.transistorsoft.locationmanager.adapter.BackgroundGeolocation.getInstance(app.android.context.getApplicationContext()).onActivityDestroy();
        this.intent = null;
    };
    Api.addListener = function (event, success, failure) {
        failure = failure || emptyFn;
        var cb;
        switch (event) {
            case 'location':
                cb = this.createLocationCallback(success, failure);
                this.getAdapter().onLocation(cb);
                break;
            case 'motionchange':
                cb = this.createMotionChangeCallback(success);
                this.getAdapter().onMotionChange(cb);
                break;
            case 'activitychange':
                cb = this.createActivityChangeCallback(success);
                this.getAdapter().onActivityChange(cb);
                break;
            case 'http':
                cb = this.createHttpCallback(success, failure);
                this.getAdapter().onHttp(cb);
                break;
            case 'geofence':
                cb = this.createGeofenceCallback(success);
                this.getAdapter().onGeofence(cb);
                break;
            case 'geofenceschange':
                cb = this.createGeofencesChangeCallback(success);
                this.getAdapter().onGeofencesChange(cb);
                break;
            case 'schedule':
                cb = this.createScheduleCallback(success);
                this.getAdapter().onSchedule(cb);
                break;
            case 'heartbeat':
                cb = this.createHeartbeatCallback(success);
                this.getAdapter().onHeartbeat(cb);
                break;
            case 'providerchange':
                cb = this.createProviderChangeCallback(success);
                this.getAdapter().onLocationProviderChange(cb);
                break;
            case 'powersavechange':
                cb = this.createPowerSaveChangeCallback(success);
                this.getAdapter().onPowerSaveChange(cb);
                break;
            case 'enabledchange':
                cb = this.createEnabledChangeCallback(success);
                this.getAdapter().onEnabledChange(cb);
                break;
            case 'connectivitychange':
                cb = this.createConnectivityChangeCallback(success);
                this.getAdapter().onConnectivityChange(cb);
                break;
        }
        return cb;
    };
    Api.removeListener = function (event, callback) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.getAdapter().removeListener(event, callback);
            resolve();
        });
    };
    Api.removeListeners = function (event) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            if (event) {
                _this.getAdapter().removeListeners(event);
            }
            else {
                _this.getAdapter().removeListeners();
            }
            resolve();
        });
    };
    /**
    * Configuration Methods
    */
    Api.ready = function (params) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var config = TSConfig.getInstance(_this.getContext());
            if (config.isFirstBoot()) {
                config.updateWithJSONObject(new org.json.JSONObject(JSON.stringify(_this.applyHeadlessJobService(params))));
            }
            else if (params.reset === true) {
                config.reset();
                config.updateWithJSONObject(new org.json.JSONObject(JSON.stringify(_this.applyHeadlessJobService(params))));
            }
            _this.getAdapter().ready(new TSCallback({
                onSuccess: function () {
                    resolve(JSON.parse(config.toJson().toString()));
                },
                onFailure: function (error) {
                    reject(error);
                }
            }));
        });
    };
    Api.configure = function (params) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var config = TSConfig.getInstance(_this.getContext());
            config.updateWithJSONObject(new org.json.JSONObject(JSON.stringify(_this.applyHeadlessJobService(params))));
            _this.getAdapter().ready(new TSCallback({
                onSuccess: function () {
                    resolve(JSON.parse(config.toJson().toString()));
                },
                onFailure: function (error) {
                    reject(error);
                }
            }));
        });
    };
    Api.setConfig = function (params) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var config = TSConfig.getInstance(_this.getContext());
            config.updateWithJSONObject(new org.json.JSONObject(JSON.stringify(_this.applyHeadlessJobService(params))));
            resolve(JSON.parse(config.toJson().toString()));
        });
    };
    Api.reset = function (params) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var config = TSConfig.getInstance(_this.getContext());
            config.reset();
            config.updateWithJSONObject(new org.json.JSONObject(JSON.stringify(_this.applyHeadlessJobService(params))));
            resolve(JSON.parse(config.toJson().toString()));
        });
    };
    Api.getState = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var config = TSConfig.getInstance(_this.getContext());
            resolve(JSON.parse(config.toJson().toString()));
        });
    };
    /**
    * Tracking Methods
    */
    Api.start = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var adapter = _this.getAdapter();
            var config = TSConfig.getInstance(_this.getContext());
            adapter.start(new TSCallback({
                onSuccess: function () {
                    resolve(JSON.parse(config.toJson().toString()));
                },
                onFailure: function (error) {
                    reject(error);
                }
            }));
        });
    };
    Api.stop = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var adapter = _this.getAdapter();
            var config = TSConfig.getInstance(_this.getContext());
            adapter.stop(new TSCallback({
                onSuccess: function () {
                    resolve(JSON.parse(config.toJson().toString()));
                },
                onFailure: function (error) {
                    reject(error);
                }
            }));
        });
    };
    Api.changePace = function (value) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var adapter = _this.getAdapter();
            adapter.changePace(value, new TSCallback({
                onSuccess: function () {
                    resolve();
                },
                onFailure: function (error) {
                    reject(error);
                }
            }));
        });
    };
    Api.startSchedule = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var config = TSConfig.getInstance(_this.getContext());
            if (_this.getAdapter().startSchedule()) {
                resolve(JSON.parse(config.toJson().toString()));
            }
            else {
                reject("Failed to start schedule.  Did you configure a #schedule?");
            }
        });
    };
    Api.stopSchedule = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var config = TSConfig.getInstance(_this.getContext());
            _this.getAdapter().stopSchedule();
            resolve(JSON.parse(config.toJson().toString()));
        });
    };
    Api.startGeofences = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var config = TSConfig.getInstance(_this.getContext());
            _this.getAdapter().startGeofences(new TSCallback({
                onSuccess: function () {
                    resolve(JSON.parse(config.toJson().toString()));
                },
                onFailure: function (error) {
                    reject(error);
                }
            }));
        });
    };
    Api.getCurrentPosition = function (options) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var builder = new TSCurrentPositionRequest.Builder(_this.getContext());
            if (typeof (options.samples) === 'number') {
                builder.setSamples(options.samples);
            }
            if (typeof (options.extras) === 'object') {
                builder.setExtras(new org.json.JSONObject(JSON.stringify(options.extras)));
            }
            if (typeof (options.persist) === 'boolean') {
                builder.setPersist(options.persist);
            }
            if (typeof (options.timeout) === 'number') {
                builder.setTimeout(options.timeout);
            }
            if (typeof (options.maximumAge) === 'number') {
                builder.setMaximumAge(new java.lang.Long(options.maximumAge));
            }
            if (typeof (options.desiredAccuracy) === 'number') {
                builder.setDesiredAccuracy(options.desiredAccuracy);
            }
            builder.setCallback(new TSLocationCallback({
                onLocation: function (tsLocation) {
                    resolve(JSON.parse(tsLocation.toJson().toString()));
                },
                onError: function (error) {
                    reject(error);
                }
            }));
            _this.getAdapter().getCurrentPosition(builder.build());
        });
    };
    Api.watchPosition = function (success, failure, options) {
        var builder = new TSWatchPositionRequest.Builder(this.getContext());
        if (typeof (options.interval) === 'number') {
            builder.setInterval(new java.lang.Long(options.interval));
        }
        if (typeof (options.extras) === 'object') {
            builder.setExtras(new org.json.JSONObject(JSON.stringify(options.extras)));
        }
        if (typeof (options.persist) === 'boolean') {
            builder.setPersist(options.persist);
        }
        if (typeof (options.desiredAccuracy) === 'number') {
            builder.setDesiredAccuracy(options.desiredAccuracy);
        }
        builder.setCallback(new TSLocationCallback({
            onLocation: function (tsLocation) {
                success(JSON.parse(tsLocation.toJson().toString()));
            },
            onError: function (error) {
                failure(error);
            }
        }));
        this.getAdapter().watchPosition(builder.build());
    };
    Api.stopWatchPosition = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.getAdapter().stopWatchPosition(new TSCallback({
                onSuccess: function () {
                    resolve();
                },
                onFailure: function (error) {
                    reject(error);
                }
            }));
        });
    };
    Api.getOdometer = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            resolve(_this.getAdapter().getOdometer());
        });
    };
    Api.setOdometer = function (value) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.getAdapter().setOdometer(new java.lang.Float(value), new TSLocationCallback({
                onLocation: function (tsLocation) {
                    resolve(JSON.parse(tsLocation.toJson().toString()));
                },
                onError: function (error) {
                    reject(error);
                }
            }));
        });
    };
    Api.resetOdometer = function (value) {
        return this.setOdometer(0);
    };
    /**
    * HTTP & Persistence Methods
    */
    Api.sync = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.getAdapter().sync(new TSSyncCallback({
                onSuccess: function (records) {
                    var size = records.size();
                    var result = [];
                    for (var i = 0; i < size; i++) {
                        var record = records.get(i);
                        result.push(JSON.parse(record.json.toString()));
                    }
                    resolve(result);
                },
                onFailure: function (error) {
                    reject(error);
                }
            }));
        });
    };
    Api.getLocations = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.getAdapter().getLocations(new TSGetLocationsCallback({
                onSuccess: function (records) {
                    var size = records.size();
                    var result = [];
                    for (var i = 0; i < size; i++) {
                        var record = records.get(i);
                        result.push(JSON.parse(record.json.toString()));
                    }
                    resolve(result);
                },
                onFailure: function (error) {
                    reject(error);
                }
            }));
        });
    };
    Api.getCount = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.getAdapter().getCount(new TSGetCountCallback({
                onSuccess: function (count) {
                    resolve(count);
                },
                onFailure: function (error) {
                    reject(error);
                }
            }));
        });
    };
    Api.insertLocation = function (data) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var callback = new TSInsertLocationCallback({
                onSuccess: function (uuid) {
                    resolve(uuid);
                },
                onFailure: function (error) {
                    reject(error);
                }
            });
            _this.getAdapter().insertLocation(new org.json.JSONObject(JSON.stringify(data)), callback);
        });
    };
    // @deprecated
    Api.clearDatabase = function () {
        return this.destroyLocations();
    };
    Api.destroyLocations = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.getAdapter().destroyLocations(new TSCallback({
                onSuccess: function () {
                    resolve();
                },
                onFailure: function (error) {
                    reject(error);
                }
            }));
        });
    };
    /**
    * Geofencing Methods
    */
    Api.addGeofence = function (params) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            try {
                _this.getAdapter().addGeofence(_this.buildGeofence(params), new TSCallback({
                    onSuccess: function () {
                        resolve();
                    },
                    onFailure: function (error) {
                        reject(error);
                    }
                }));
            }
            catch (error) {
                reject(error.getMessage());
            }
        });
    };
    Api.removeGeofence = function (identifier) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.getAdapter().removeGeofence(identifier, new TSCallback({
                onSuccess: function () {
                    resolve();
                },
                onFailure: function (error) {
                    reject(error);
                }
            }));
        });
    };
    Api.addGeofences = function (geofences) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var list = new java.util.ArrayList();
            for (var n = 0, len = geofences.length; n < len; n++) {
                try {
                    list.add(_this.buildGeofence(geofences[n]));
                }
                catch (error) {
                    reject(error.getMessage());
                    return;
                }
            }
            _this.getAdapter().addGeofences(list, new TSCallback({
                onSuccess: function () {
                    resolve();
                },
                onFailure: function (error) {
                    reject(error);
                }
            }));
        });
    };
    Api.buildGeofence = function (params) {
        var builder = new TSGeofence.Builder();
        builder.setIdentifier(params.identifier);
        builder.setLatitude(params.latitude);
        builder.setLongitude(params.longitude);
        builder.setRadius(params.radius);
        if (typeof (params.notifyOnEntry) === 'boolean') {
            builder.setNotifyOnEntry(params.notifyOnEntry);
        }
        if (typeof (params.notifyOnExit) === 'boolean') {
            builder.setNotifyOnExit(params.notifyOnExit);
        }
        if (typeof (params.notifyOnDwell) === 'boolean') {
            builder.setNotifyOnDwell(params.notifyOnDwell);
        }
        if (typeof (params.loiteringDelay) === 'number') {
            builder.setLoiteringDelay(params.loiteringDelay);
        }
        if (typeof (params.extras) === 'object') {
            builder.setExtras(new org.json.JSONObject(JSON.stringify(params.extras)));
        }
        return builder.build();
    };
    Api.removeGeofences = function (geofences) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            // Handle case where no geofences are provided (ie: remove all geofences).
            geofences = geofences || [];
            var identifiers = new java.util.ArrayList();
            geofences.forEach(function (identifier) { identifiers.add(identifier); });
            _this.getAdapter().removeGeofences(identifiers, new TSCallback({
                onSuccess: function () {
                    resolve();
                },
                onFailure: function (error) {
                    reject(error);
                }
            }));
        });
    };
    Api.getGeofences = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.getAdapter().getGeofences(new TSGetGeofencesCallback({
                onSuccess: function (records) {
                    var size = records.size();
                    var result = [];
                    for (var i = 0; i < size; i++) {
                        var geofence = records.get(i);
                        result.push(JSON.parse(geofence.toJson()));
                    }
                    resolve(result);
                },
                onFailure: function (error) {
                    reject(error);
                }
            }));
        });
    };
    /**
    * Logging & Debug methods
    */
    Api.getLog = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.getAdapter().getLog(new TSGetLogCallback({
                onSuccess: function (log) {
                    resolve(log);
                },
                onFailure: function (error) {
                    reject(error);
                }
            }));
        });
    };
    Api.emailLog = function (email) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.getAdapter().emailLog(email, app.android.foregroundActivity, new TSEmailLogCallback({
                onSuccess: function () {
                    resolve();
                },
                onFailure: function (error) {
                    reject(error);
                }
            }));
        });
    };
    Api.destroyLog = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.getAdapter().destroyLog(new TSCallback({
                onSuccess: function () {
                    resolve();
                },
                onFailure: function (error) {
                    reject(error);
                }
            }));
        });
    };
    Api.getSensors = function () {
        return new Promise(function (resolve, reject) {
            var sensors = TSSensors.getInstance(app.android.context);
            var result = {
                "platform": "android",
                "accelerometer": sensors.hasAccelerometer(),
                "magnetometer": sensors.hasMagnetometer(),
                "gyroscope": sensors.hasGyroscope(),
                "significant_motion": sensors.hasSignificantMotion()
            };
            resolve(result);
        });
    };
    Api.isPowerSaveMode = function (success, failure) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var isPowerSaveMode = _this.getAdapter().isPowerSaveMode();
            resolve(isPowerSaveMode);
        });
    };
    Api.startBackgroundTask = function (success) {
        // Just return 0 for compatibility with iOS API.  Android has no concept of these iOS-only background-tasks.
        return new Promise(function (resolve, reject) {
            resolve(0);
        });
    };
    Api.finish = function (taskId) {
        // Just an empty function for compatibility with iOS.  Android has no concept of these iOS-only background-tasks.
    };
    Api.playSound = function (soundId) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.getAdapter().startTone(soundId);
            resolve();
        });
    };
    Api.log = function (level, msg) {
        TSLog.log(level, msg);
    };
    Api.createLocationCallback = function (success, failure) {
        failure = failure || emptyFn;
        return new TSLocationCallback({
            onLocation: function (tsLocation) {
                success(JSON.parse(tsLocation.toJson().toString()));
            },
            onError: function (error) {
                failure(error);
            }
        });
    };
    Api.createMotionChangeCallback = function (callback) {
        return new TSLocationCallback({
            onLocation: function (tsLocation) {
                var isMoving = tsLocation.getIsMoving();
                callback(isMoving, JSON.parse(tsLocation.toJson().toString()));
            },
            onError: function (error) {
            }
        });
    };
    Api.createHttpCallback = function (success, failure) {
        return new TSHttpResponseCallback({
            onHttpResponse: function (response) {
                var params = {
                    success: response.isSuccess().booleanValue(),
                    status: response.status,
                    responseText: response.responseText
                };
                if (response.isSuccess().booleanValue()) {
                    success(params);
                }
                else {
                    failure(params);
                }
            }
        });
    };
    Api.createActivityChangeCallback = function (callback) {
        return new TSActivityChangeCallback({
            onActivityChange: function (event) {
                callback(JSON.parse(event.toJson().toString()));
            }
        });
    };
    Api.createGeofenceCallback = function (callback) {
        return new TSGeofenceCallback({
            onGeofence: function (event) {
                callback(JSON.parse(event.toJson().toString()));
            }
        });
    };
    Api.createGeofencesChangeCallback = function (callback) {
        return new TSGeofencesChangeCallback({
            onGeofencesChange: function (event) {
                callback(JSON.parse(event.toJson().toString()));
            }
        });
    };
    Api.createScheduleCallback = function (callback) {
        return new TSScheduleCallback({
            onSchedule: function (event) {
                callback(JSON.parse(event.toJson().toString()));
            }
        });
    };
    Api.createProviderChangeCallback = function (callback) {
        return new TSLocationProviderChangeCallback({
            onLocationProviderChange: function (event) {
                callback(JSON.parse(event.toJson().toString()));
            }
        });
    };
    Api.createHeartbeatCallback = function (callback) {
        return new TSHeartbeatCallback({
            onHeartbeat: function (event) {
                callback(JSON.parse(event.toJson().toString()));
            }
        });
    };
    Api.createPowerSaveChangeCallback = function (callback) {
        return new TSPowerSaveChangeCallback({
            onPowerSaveChange: function (isPowerSaveMode) {
                callback(isPowerSaveMode);
            }
        });
    };
    Api.createEnabledChangeCallback = function (callback) {
        return new TSEnabledChangeCallback({
            onEnabledChange: function (enabled) {
                console.log('- enabledchange: ', enabled);
                callback({ enabled: enabled });
            }
        });
    };
    Api.createConnectivityChangeCallback = function (callback) {
        return new TSConnectivityChangeCallback({
            onConnectivityChange: function (event) {
                callback({ connected: event.hasConnection() });
            }
        });
    };
    Api.applyHeadlessJobService = function (params) {
        params.headlessJobService = HEADLESS_JOB_SERVICE;
        return params;
    };
    Api.init = function () {
        var _this = this;
        if (!app.android.startActivity || (this.intent !== null)) {
            return;
        }
        this.intent = app.android.startActivity.getIntent();
        // Handle Google Play Services errors
        this.getAdapter().onPlayServicesConnectError(new TSPlayServicesConnectErrorCallback({
            onPlayServicesConnectError: function (errorCode) {
                _this.handleGooglePlayServicesConnectError(errorCode);
            }
        }));
        var config = TSConfig.getInstance(this.getContext());
        config.useCLLocationAccuracy(new java.lang.Boolean(true));
    };
    Api.getContext = function () {
        return app.android.context;
    };
    Api.getIntent = function () {
        var activity = (app.android.startActivity) ? app.android.startActivity : app.android.foregroundActivity;
        return (activity) ? activity.getIntent() : this.intent;
    };
    Api.getAdapter = function () {
        if (!this.intent) {
            this.init();
        }
        return com.transistorsoft.locationmanager.adapter.BackgroundGeolocation.getInstance(app.android.context.getApplicationContext(), this.getIntent());
    };
    Api.handleGooglePlayServicesConnectError = function (errorCode) {
        com.google.android.gms.common.GoogleApiAvailability.getInstance().getErrorDialog(app.android.foregroundActivity, errorCode, 1001).show();
    };
    Api.intent = null;
    return Api;
}());
background_geolocation_1.BackgroundGeolocation.mountNativeApi(Api);
