"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var background_geolocation_1 = require("./background-geolocation");
exports.BackgroundGeolocation = background_geolocation_1.BackgroundGeolocation;
var utils = require("utils/utils");
var TS_LOCATION_TYPE_MOTIONCHANGE = 0;
var TS_LOCATION_TYPE_CURRENT = 1;
var TS_LOCATION_TYPE_SAMPLE = 2;
var emptyFn = function () { };
var Api = /** @class */ (function () {
    function Api() {
    }
    /**
    * Configuration Methods
    */
    Api.addListener = function (event, success, failure) {
        var _this = this;
        var callback;
        failure = failure || emptyFn;
        switch (event) {
            case 'location':
                callback = function (tsLocation) {
                    var location = _this.getJsObjectFromNSDictionary(tsLocation.toDictionary());
                    success(location);
                };
                this.getAdapter().onLocationFailure(callback, failure);
                break;
            case 'motionchange':
                callback = function (tsLocation) {
                    var location = _this.getJsObjectFromNSDictionary(tsLocation.toDictionary());
                    var isMoving = tsLocation.isMoving;
                    success(isMoving, location);
                };
                this.getAdapter().onMotionChange(callback);
                break;
            case 'activitychange':
                callback = function (event) {
                    var params = { activity: event.activity, confidence: event.confidence };
                    success(params);
                };
                this.getAdapter().onActivityChange(callback);
                break;
            case 'heartbeat':
                callback = function (event) {
                    var location = _this.getJsObjectFromNSDictionary(event.location.toDictionary());
                    var params = { location: location };
                    success(params);
                };
                this.getAdapter().onHeartbeat(callback);
                break;
            case 'geofence':
                callback = function (event) {
                    var params = event.toDictionary().mutableCopy();
                    params.setObjectForKey(event.location.toDictionary(), "location");
                    success(_this.getJsObjectFromNSDictionary(params));
                };
                this.getAdapter().onGeofence(callback);
                break;
            case 'geofenceschange':
                callback = function (event) {
                    var params = _this.getJsObjectFromNSDictionary(event.toDictionary());
                    success(params);
                };
                this.getAdapter().onGeofencesChange(callback);
                break;
            case 'http':
                callback = function (event) {
                    var params = {
                        "success": event.isSuccess,
                        "status": event.statusCode,
                        "responseText": event.responseText
                    };
                    if (event.isSuccess) {
                        success(params);
                    }
                    else {
                        failure(params);
                    }
                };
                this.getAdapter().onHttp(callback);
                break;
            case 'providerchange':
                callback = function (event) {
                    var params = _this.getJsObjectFromNSDictionary(event.toDictionary());
                    success(params);
                };
                this.getAdapter().onProviderChange(callback);
                break;
            case 'schedule':
                callback = function (event) {
                    var state = _this.getJsObjectFromNSDictionary(event.state);
                    success(state);
                };
                this.getAdapter().onSchedule(callback);
                break;
            case 'powersavechange':
                callback = function (event) {
                    success(event.isPowerSaveMode);
                };
                this.getAdapter().onPowerSaveChange(callback);
                break;
            case 'enabledchange':
                callback = function (event) {
                    var params = {
                        enabled: event.enabled
                    };
                    success(params);
                };
                this.getAdapter().onEnabledChange(callback);
                break;
            case 'connectivitychange':
                callback = function (event) {
                    var params = {
                        connected: event.hasConnection
                    };
                    success(params);
                };
                this.getAdapter().onConnectivityChange(callback);
                break;
        }
        return callback;
    };
    Api.removeListener = function (event, callback) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.getAdapter().removeListenerCallback(event, callback);
            // It's not possible to remove single event-listener with NativeScript since the native block is wrapped in a Javascript function
            reject("BackgroundGeolocation#removeListener - Unfortunately it's not possible to remove a single event-listener with NativeScript.  You can only remove ALL listeners for a particular event, eg: removeListeners('location')");
        });
    };
    Api.removeListeners = function (event) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            if (event) {
                _this.getAdapter().removeListenersForEvent(event);
            }
            else {
                _this.getAdapter().removeListeners();
            }
            resolve();
        });
    };
    Api.ready = function (params) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var config = TSConfig.sharedInstance();
            if (config.isFirstBoot) {
                config.updateWithDictionary(params);
            }
            else if (params.reset === true) {
                config.reset();
                config.updateWithDictionary(params);
            }
            var locationManager = _this.getAdapter();
            locationManager.ready();
            resolve(_this.getJsObjectFromNSDictionary(config.toDictionary()));
        });
    };
    Api.configure = function (params) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var config = TSConfig.sharedInstance();
            var locationManager = _this.getAdapter();
            locationManager.configure(params);
            resolve(_this.getJsObjectFromNSDictionary(config.toDictionary()));
        });
    };
    Api.setConfig = function (params) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var config = TSConfig.sharedInstance();
            config.updateWithDictionary(params);
            resolve(_this.getJsObjectFromNSDictionary(config.toDictionary()));
        });
    };
    Api.reset = function (params) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var config = TSConfig.sharedInstance();
            config.reset();
            config.updateWithDictionary(params);
            resolve(_this.getJsObjectFromNSDictionary(config.toDictionary()));
        });
    };
    Api.getState = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var config = TSConfig.sharedInstance();
            resolve(_this.getJsObjectFromNSDictionary(config.toDictionary()));
        });
    };
    /**
    * Tracking Methods
    */
    Api.start = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.getAdapter().start();
            var config = TSConfig.sharedInstance();
            resolve(_this.getJsObjectFromNSDictionary(config.toDictionary()));
        });
    };
    Api.stop = function (success, failure) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.getAdapter().stop();
            var config = TSConfig.sharedInstance();
            resolve(_this.getJsObjectFromNSDictionary(config.toDictionary()));
        });
    };
    Api.startGeofences = function (success, failure) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.getAdapter().startGeofences();
            var config = TSConfig.sharedInstance();
            resolve(_this.getJsObjectFromNSDictionary(config.toDictionary()));
        });
    };
    Api.changePace = function (value, success, failure) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.getAdapter().changePace(value);
            resolve(value);
        });
    };
    Api.startSchedule = function (success, failure) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.getAdapter().startSchedule();
            var config = TSConfig.sharedInstance();
            resolve(_this.getJsObjectFromNSDictionary(config.toDictionary()));
        });
    };
    Api.stopSchedule = function (success, failure) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.getAdapter().stopSchedule();
            var config = TSConfig.sharedInstance();
            resolve(_this.getJsObjectFromNSDictionary(config.toDictionary()));
        });
    };
    Api.getCurrentPosition = function (options) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var request = new TSCurrentPositionRequest();
            request.success = function (tsLocation) {
                resolve(_this.getJsObjectFromNSDictionary(tsLocation.toDictionary()));
            };
            request.failure = function (error) {
                reject(error);
            };
            if (typeof (options.timeout) === 'number') {
                request.timeout = options.timeout;
            }
            if (typeof (options.maximumAge) === 'number') {
                request.maximumAge = options.maximumAge;
            }
            if (typeof (options.persist) === 'boolean') {
                request.persist = options.persist;
            }
            if (typeof (options.samples) === 'number') {
                request.samples = options.samples;
            }
            if (typeof (options.desiredAccuracy) === 'number') {
                request.desiredAccuracy = options.desiredAccuracy;
            }
            if (typeof (options.extras) === 'object') {
                request.extras = options.extras;
            }
            _this.getAdapter().getCurrentPosition(request);
        });
    };
    Api.watchPosition = function (success, failure, options) {
        var _this = this;
        var request = new TSWatchPositionRequest();
        request.success = function (tsLocation) {
            success(_this.getJsObjectFromNSDictionary(tsLocation.toDictionary()));
        };
        request.failure = function (error) {
            failure(error);
        };
        if (typeof (options.interval) === 'number') {
            request.interval = options.interval;
        }
        if (typeof (options.desiredAccuracy) === 'number') {
            request.desiredAccuracy = options.desiredAccuracy;
        }
        if (typeof (options.persist) === 'boolean') {
            request.persist = options.persist;
        }
        if (typeof (options.extras) === 'object') {
            request.extras = options.extras;
        }
        if (typeof (options.timeout) === 'number') {
            request.timeout = options.timeout;
        }
        this.getAdapter().watchPosition(request);
    };
    Api.stopWatchPosition = function (success, failure) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.getAdapter().stopWatchPosition();
            resolve(true);
        });
    };
    Api.getOdometer = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            resolve(_this.getAdapter().getOdometer());
        });
    };
    Api.setOdometer = function (value) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var request = new TSCurrentPositionRequest();
            request.success = function (tsLocation) {
                resolve(_this.getJsObjectFromNSDictionary(tsLocation.toDictionary()));
            };
            request.failure = function (error) { reject(error); };
            _this.getAdapter().setOdometerRequest(value, request);
        });
    };
    Api.resetOdometer = function () {
        return this.setOdometer(0);
    };
    /**
    * HTTP & Persistence Methods
    */
    Api.sync = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.getAdapter().syncFailure(function (records) {
                resolve(_this.getJsArrayFromNSArray(records));
            }, function (error) {
                reject(error.code);
            });
        });
    };
    Api.getLocations = function (success, failure) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.getAdapter().getLocationsFailure(function (rs) {
                resolve(_this.getJsArrayFromNSArray(rs));
            }, function (error) {
                reject(error);
            });
        });
    };
    Api.getCount = function (success) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            resolve(_this.getAdapter().getCount());
        });
    };
    Api.insertLocation = function (data) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.getAdapter().insertLocationSuccessFailure(data, function (uuid) {
                resolve(uuid);
            }, function (error) {
                reject(error);
            });
        });
    };
    // @deprecated
    Api.clearDatabase = function () {
        return this.destroyLocations();
    };
    Api.destroyLocations = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.getAdapter().destroyLocationsFailure(function () {
                resolve();
            }, function (error) {
                reject(error);
            });
        });
    };
    /**
    * Geofencing Methods
    */
    Api.addGeofence = function (params) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var success = function () { resolve(); };
            var failure = function (error) { reject(error); };
            var geofence = _this.buildGeofence(params);
            if (geofence) {
                _this.getAdapter().addGeofenceSuccessFailure(geofence, success, failure);
            }
            else {
                reject('Invalid geofence data');
            }
        });
    };
    Api.removeGeofence = function (identifier) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var success = function () { resolve(); };
            var failure = function (error) { reject(error); };
            _this.getAdapter().removeGeofenceSuccessFailure(identifier, success, failure);
        });
    };
    Api.addGeofences = function (geofences) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var rs = [];
            for (var n = 0, len = geofences.length; n < len; n++) {
                var geofence = _this.buildGeofence(geofences[n]);
                if (geofence != null) {
                    rs.push(geofence);
                }
                else {
                    reject('Invalid geofence data: ' + JSON.stringify(geofences[n]));
                    return;
                }
            }
            var success = function () { resolve(); };
            var failure = function (error) { reject(error); };
            _this.getAdapter().addGeofencesSuccessFailure(rs, success, failure);
        });
    };
    Api.removeGeofences = function (geofences) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var success = function () { resolve(); };
            var failure = function (error) { reject(error); };
            _this.getAdapter().removeGeofencesSuccessFailure(geofences, success, failure);
        });
    };
    Api.getGeofences = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.getAdapter().getGeofencesFailure(function (geofences) {
                var rs = [];
                for (var loop = 0; loop < geofences.count; loop++) {
                    var geofence = geofences.objectAtIndex(loop);
                    rs.push(_this.getJsObjectFromNSDictionary(geofence.toDictionary()));
                }
                resolve(rs);
            }, function (error) {
                reject(error);
            });
        });
    };
    Api.startBackgroundTask = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            resolve(_this.getAdapter().createBackgroundTask());
        });
    };
    Api.finish = function (taskId) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.getAdapter().stopBackgroundTask(taskId);
            resolve();
        });
    };
    /**
    * Logging & Debug methods
    */
    Api.playSound = function (soundId) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.getAdapter().playSound(soundId);
            resolve();
        });
    };
    Api.getLog = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var success = function (log) { resolve(log); };
            var failure = function (error) { reject(error); };
            _this.getAdapter().getLogFailure(success, failure);
        });
    };
    Api.destroyLog = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            if (_this.getAdapter().destroyLog()) {
                resolve();
            }
            else {
                reject();
            }
        });
    };
    Api.emailLog = function (email) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var success = function () { resolve(); };
            var failure = function (error) { reject(error); };
            _this.getAdapter().emailLogSuccessFailure(email, success, failure);
        });
    };
    Api.getSensors = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var adapter = _this.getAdapter();
            var result = {
                "platform": "ios",
                "accelerometer": adapter.isAccelerometerAvailable(),
                "gyroscope": adapter.isGyroAvailable(),
                "magnetometer": adapter.isMagnetometerAvailable(),
                "motion_hardware": adapter.isMotionHardwareAvailable()
            };
            resolve(result);
        });
    };
    Api.isPowerSaveMode = function (success, failure) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            resolve(_this.getAdapter().isPowerSaveMode());
        });
    };
    Api.log = function (level, msg) {
        this.getAdapter().logMessage(level, msg);
    };
    /**
    * Private
    */
    Api.buildGeofence = function (params) {
        if (!params.identifier || !params.radius || !params.latitude || !params.longitude) {
            return null;
        }
        var geofence = new TSGeofence();
        geofence.identifier = params.identifier;
        geofence.radius = params.radius;
        geofence.latitude = params.latitude;
        geofence.longitude = params.longitude;
        if (typeof (params.notifyOnEntry) === 'boolean') {
            geofence.notifyOnEntry = params.notifyOnEntry;
        }
        if (typeof (params.notifyOnExit) === 'boolean') {
            geofence.notifyOnExit = params.notifyOnExit;
        }
        if (typeof (params.notifyOnDwell) === 'boolean') {
            geofence.notifyOnDwell = params.notifyOnDwell;
        }
        if (typeof (params.loiteringDelay) === 'number') {
            geofence.loiteringDelay = params.loiteringDelay;
        }
        if (typeof (params.extras) === 'object') {
            geofence.extras = params.extras;
        }
        return geofence;
    };
    Api.getAdapter = function () {
        if (!this.adapter) {
            var app = utils.ios.getter(UIApplication, UIApplication.sharedApplication);
            this.adapter = TSLocationManager.sharedInstance();
            this.adapter.viewController = app.keyWindow.rootViewController;
        }
        return this.adapter;
    };
    Api.getJsObjectFromNSDictionary = function (dictionary) {
        var keys = dictionary.allKeys;
        var result = {};
        for (var loop = 0; loop < keys.count; loop++) {
            var key = keys[loop];
            var item = dictionary.objectForKey(key);
            result[key] = this.getJsObject(item);
        }
        return result;
    };
    Api.getJsArrayFromNSArray = function (array) {
        var result = [];
        for (var loop = 0; loop < array.count; loop++) {
            result.push(this.getJsObject(array.objectAtIndex(loop)));
        }
        return result;
    };
    Api.getJsObject = function (object) {
        if (object instanceof NSDictionary) {
            return this.getJsObjectFromNSDictionary(object);
        }
        if (object instanceof NSArray) {
            return this.getJsArrayFromNSArray(object);
        }
        return object;
    };
    return Api;
}());
background_geolocation_1.BackgroundGeolocation.mountNativeApi(Api);
