"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Platform = require("platform");
var TAG = "TSLocationManager";
var emptyFn = function () { };
/**
* Url to transistorsoft tracking test server.  Configure the plugin with a #username to automatically post
* locations to the test server, eg:
{
  username: 'my-username'
}
* View your tracking online by visiting:
* http://tracker.transistorsoft.com/username
*

    "build": "tsc",
    "demo.ios": "npm run preparedemo && cd demo && tns debug ios --emulator",
    "demo.android": "npm run preparedemo && cd demo && tns debug android",
    "preparedemo": "npm run build && cd demo && tns plugin remove nativescript-background-geolocation-lt && tns plugin add ../src",
    "setup": "cd ../demo && npm install && cd .. && npm run build && cd demo && tns plugin add ../src && cd ..",
    "fix-xcode": "xcproj --project 'demo/platforms/ios/demo.xcodeproj' touch && xcproj --project 'demo/platforms/ios/Pods/Pods.xcodeproj' touch",

*/
var TEST_SERVER_URL = 'http://tracker.transistorsoft.com/locations/';
/**
* Logger
*/
var Logger = /** @class */ (function () {
    function Logger(api) {
        this.api = api;
    }
    Logger.prototype.error = function (msg) {
        this.log('error', msg);
    };
    Logger.prototype.warn = function (msg) {
        this.log('warn', msg);
    };
    Logger.prototype.debug = function (msg) {
        this.log('debug', msg);
    };
    Logger.prototype.notice = function (msg) {
        this.log('notice', msg);
    };
    Logger.prototype.header = function (msg) {
        this.log('header', msg);
    };
    Logger.prototype.on = function (msg) {
        this.log('on', msg);
    };
    Logger.prototype.off = function (msg) {
        this.log('off', msg);
    };
    Logger.prototype.ok = function (msg) {
        this.log('ok', msg);
    };
    Logger.prototype.log = function (level, msg) {
        this.api.log(level, msg);
    };
    return Logger;
}());
exports.Logger = Logger;
/**
* BackgroundGeolocation
*/
var BackgroundGeolocation = /** @class */ (function () {
    function BackgroundGeolocation() {
    }
    BackgroundGeolocation.mountNativeApi = function (api) {
        this.api = api;
        this.logger = new Logger(api);
    };
    BackgroundGeolocation.registerHeadlessTask = function (callback) {
        this.headlessTask = callback;
    };
    BackgroundGeolocation.invokeHeadlessTask = function (event, completionHandler) {
        if (!this.headlessTask) {
            console.log('[BackgroundGeolocation] invokeHeadlessTask ERROR - headlessTask is null.  Did you BackgroundGeolocation.registerHeadlessTask(myTask) in your app.ts?');
            completionHandler();
            return;
        }
        this.headlessTask(event, completionHandler);
    };
    /**
    * Core Plugin Control Methods
    */
    BackgroundGeolocation.ready = function (config, success, failure) {
        config = this.validate(config);
        if (arguments.length <= 1) {
            return this.api.ready(config || {});
        }
        else {
            this.api.ready(config).then(success).catch(failure);
        }
    };
    /**
    * Reset plugin confg to default
    */
    BackgroundGeolocation.reset = function (config, success, failure) {
        if ((typeof (config) === 'function') || (typeof (success) === 'function')) {
            if (typeof (config) === 'function') {
                success = config;
                config = {};
            }
            config = this.validate(config || {});
            this.api.reset(config).then(success).catch(failure);
        }
        else {
            return this.api.reset(this.validate(config || {}));
        }
    };
    /**
    * Perform initial configuration of plugin.  Reset config to default before applying supplied configuration
    */
    BackgroundGeolocation.configure = function (config, success, failure) {
        config = this.validate(config);
        if (arguments.length == 1) {
            return this.api.configure(config);
        }
        else {
            this.api.configure(config).then(success).catch(failure);
        }
    };
    /**
    * Listen to a plugin event
    */
    BackgroundGeolocation.addListener = function (event, success, failure) {
        if (this.EVENTS.indexOf(event) < 0) {
            throw "BackgroundGeolocation#on - Unknown event '" + event + "'";
        }
        var nativeCallback = this.api.addListener.apply(this.api, arguments);
        if (nativeCallback) {
            this.registerCallback(event, success, nativeCallback);
        }
    };
    BackgroundGeolocation.registerCallback = function (event, clientCallback, nativeCallback) {
        this.listeners[event].push({
            clientCallback: clientCallback,
            nativeCallback: nativeCallback
        });
    };
    // @alias #addListener
    BackgroundGeolocation.on = function (event, success, failure) {
        this.addListener.apply(this, arguments);
    };
    /**
    * Remove a single plugin event-listener, supplying a reference to the handler initially supplied to #un
    */
    BackgroundGeolocation.removeListener = function (event, clientCallback, success, failure) {
        if (this.EVENTS.indexOf(event) < 0) {
            throw "BackgroundGeolocation#un - Unknown event '" + event + "'";
        }
        var listeners = this.listeners[event];
        var listener = listeners.find(function (i) { return i.clientCallback === clientCallback; });
        if (listener) {
            listeners.splice(listeners.indexOf(listener), 1);
            if (arguments.length == 2) {
                return this.api.removeListener(event, listener.nativeCallback);
            }
            else {
                this.api.removeListener(event, listener.nativeCallback).then(success).catch(failure);
            }
        }
        else {
            var error_1 = 'Failed to removeListener for event: ' + event;
            failure = failure || emptyFn;
            console.warn(error_1);
            if (arguments.length == 2) {
                return new Promise(function (resolve, reject) {
                    reject(error_1);
                });
            }
            else {
                failure(error_1);
            }
        }
    };
    // @alias #removeListener
    BackgroundGeolocation.un = function (event, handler, success, failiure) {
        this.removeListener.apply(this, arguments);
    };
    /**
    * Remove all event listeners
    */
    BackgroundGeolocation.removeListeners = function (event, success, failure) {
        if (!arguments.length) {
            return this.api.removeListeners(); // #removeListeners()
        }
        else if (typeof (arguments[0]) === 'string') {
            if (this.EVENTS.indexOf(event) < 0) {
                throw "BackgroundGeolocation#removeListeners - Unknown event: " + event;
            }
            // Clear client-calllbacks.
            this.listeners[event] = [];
            if (typeof (arguments[1]) === 'function') { // #removeListeners(event, success, failure);
                this.api.removeListeners(event).then(success).catch(failure);
            }
            else { // #removeListeners(event)
                return this.api.removeListeners(event);
            }
        }
        else if (typeof (arguments[0]) === 'function') { // #removeListeners(success, failure)
            success = event;
            failure = success;
            this.api.removeListeners().then(success).catch(failure);
        }
    };
    /**
    * Fetch current plugin configuration
    */
    BackgroundGeolocation.getState = function (success, failure) {
        if (!arguments.length) {
            return this.api.getState();
        }
        else {
            this.api.getState().then(success).catch(failure);
        }
    };
    /**
    * Start the plugin
    */
    BackgroundGeolocation.start = function (success, failure) {
        if (!arguments.length) {
            return this.api.start();
        }
        else {
            this.api.start().then(success).catch(failure);
        }
    };
    /**
    * Stop the plugin
    */
    BackgroundGeolocation.stop = function (success, failure) {
        if (!arguments.length) {
            return this.api.stop();
        }
        else {
            this.api.stop().then(success).catch(failure);
        }
    };
    /**
    * Start the scheduler
    */
    BackgroundGeolocation.startSchedule = function (success, failure) {
        if (!arguments.length) {
            return this.api.startSchedule();
        }
        else {
            this.api.startSchedule().then(success).catch(failure);
        }
    };
    /**
    * Stop the scheduler
    */
    BackgroundGeolocation.stopSchedule = function (success, failure) {
        if (!arguments.length) {
            return this.api.stopSchedule();
        }
        else {
            this.api.stopSchedule().then(success).catch(failure);
        }
    };
    /**
    * Initiate geofences-only mode
    */
    BackgroundGeolocation.startGeofences = function (success, failure) {
        if (!arguments.length) {
            return this.api.startGeofences();
        }
        else {
            this.api.startGeofences().then(success).catch(failure);
        }
    };
    /**
    * Start an iOS background-task, provding 180s of background running time
    */
    BackgroundGeolocation.startBackgroundTask = function (success, failure) {
        if (!arguments.length) {
            return this.api.startBackgroundTask();
        }
        else {
            if (typeof (success) !== 'function') {
                throw TAG + "#startBackgroundTask must be provided with a callback to recieve the taskId";
            }
            this.api.startBackgroundTask().then(success).catch(failure);
        }
    };
    /**
    * Signal to iOS that your background-task from #startBackgroundTask is complete
    */
    BackgroundGeolocation.finish = function (taskId, success, failure) {
        // No taskId?  Ignore it.
        if (arguments.length == 1) {
            return this.api.finish(taskId);
        }
        else {
            this.api.finish(taskId).then(success).catch(failure);
        }
    };
    /**
    * Toggle motion-state between stationary <-> moving
    */
    BackgroundGeolocation.changePace = function (isMoving, success, failure) {
        if (arguments.length == 1) {
            return this.api.changePace(isMoving);
        }
        else {
            this.api.changePace(isMoving).then(success).catch(failure);
        }
    };
    /**
    * Provide new configuration to the plugin.  This configuration will be *merged* to current configuration
    */
    BackgroundGeolocation.setConfig = function (config, success, failure) {
        if (arguments.length == 1) {
            return this.api.setConfig(config);
        }
        else {
            this.api.setConfig(config).then(success).catch(failure);
        }
    };
    /**
    * HTTP & Persistence
    *
    */
    BackgroundGeolocation.getLocations = function (success, failure) {
        if (!arguments.length) {
            return this.api.getLocations();
        }
        else {
            this.api.getLocations().then(success).catch(failure);
        }
    };
    /**
    * Fetch the current count of location records in database
    */
    BackgroundGeolocation.getCount = function (success, failure) {
        if (!arguments.length) {
            return this.api.getCount();
        }
        else {
            this.api.getCount().then(success).catch(failure);
        }
    };
    /**
    * Destroy all records in locations database
    */
    BackgroundGeolocation.destroyLocations = function (success, failure) {
        if (!arguments.length) {
            return this.api.destroyLocations();
        }
        else {
            this.api.destroyLocations().then(success).catch(failure);
        }
    };
    // @deprecated
    BackgroundGeolocation.clearDatabase = function (success, failure) {
        return this.destroyLocations.apply(this, arguments);
    };
    /**
    * Insert a single record into locations database
    */
    BackgroundGeolocation.insertLocation = function (location, success, failure) {
        if (arguments.length == 1) {
            return this.api.insertLocation(location);
        }
        else {
            this.api.insertLocation(location).then(success).catch(failure);
        }
    };
    /**
    * Manually initiate an HTTP sync operation
    */
    BackgroundGeolocation.sync = function (success, failure) {
        if (!arguments.length) {
            return this.api.sync();
        }
        else {
            this.api.sync().then(success).catch(failure);
        }
    };
    /**
    * Fetch the current value of odometer
    */
    BackgroundGeolocation.getOdometer = function (success, failure) {
        if (!arguments.length) {
            return this.api.getOdometer();
        }
        else {
            this.api.getOdometer().then(success).catch(failure);
        }
    };
    /**
    * Set the value of the odometer
    */
    BackgroundGeolocation.setOdometer = function (value, success, failure) {
        if (arguments.length == 1) {
            return this.api.setOdometer(value);
        }
        else {
            this.api.setOdometer(value).then(success).catch(failure);
        }
    };
    /**
    * Reset the value of odometer to 0
    */
    BackgroundGeolocation.resetOdometer = function (success, failure) {
        if (!arguments.length) {
            return this.api.setOdometer(0);
        }
        else {
            this.api.setOdometer(0).then(success).catch(failure);
        }
    };
    /**
    * Geofencing Methods
    */
    /**
    * Add a single geofence
    */
    BackgroundGeolocation.addGeofence = function (config, success, failure) {
        if (arguments.length == 1) {
            return this.api.addGeofence(config);
        }
        else {
            this.api.addGeofence(config).then(success).catch(failure);
        }
    };
    /**
    * Remove a single geofence by identifier
    */
    BackgroundGeolocation.removeGeofence = function (identifier, success, failure) {
        if (arguments.length == 1) {
            return this.api.removeGeofence(identifier);
        }
        else {
            this.api.removeGeofence(identifier).then(success).catch(failure);
        }
    };
    /**
    * Add a list of geofences
    */
    BackgroundGeolocation.addGeofences = function (geofences, success, failure) {
        if (arguments.length == 1) {
            return this.api.addGeofences(geofences);
        }
        else {
            this.api.addGeofences(geofences).then(success).catch(failure);
        }
    };
    /**
    * Remove geofences.  You may either supply an array of identifiers or nothing to destroy all geofences.
    * 1. removeGeofences() <-- Promise
    * 2. removeGeofences(['foo'])  <-- Promise
    *
    * 3. removeGeofences(success, [failure])
    * 4. removeGeofences(['foo'], success, [failure])
    */
    BackgroundGeolocation.removeGeofences = function (success, failure) {
        if (!arguments.length) {
            return this.api.removeGeofences();
        }
        else {
            this.api.removeGeofences().then(success).catch(failure);
        }
    };
    /**
    * Fetch all geofences from database
    */
    BackgroundGeolocation.getGeofences = function (success, failure) {
        if (!arguments.length) {
            return this.api.getGeofences();
        }
        else {
            this.api.getGeofences().then(success).catch(failure);
        }
    };
    /**
    * Fetch the current position from location-services
    */
    BackgroundGeolocation.getCurrentPosition = function (success, failure, options) {
        if (typeof (success) == 'function') {
            if (typeof (failure) == 'object') {
                options = failure;
                failure = emptyFn;
            }
            options = options || {};
            this.api.getCurrentPosition(options).then(success).catch(failure);
        }
        else {
            options = success || {};
            return this.api.getCurrentPosition(options);
        }
    };
    /**
    * Begin watching a stream of locations
    */
    BackgroundGeolocation.watchPosition = function (success, failure, options) {
        this.api.watchPosition(success, failure || emptyFn, options || {});
    };
    /**
    * Stop watching location
    */
    BackgroundGeolocation.stopWatchPosition = function (success, failure) {
        if (!arguments.length) {
            return this.api.stopWatchPosition();
        }
        else {
            this.api.stopWatchPosition().then(success).catch(failure);
        }
    };
    /**
    * Set the logLevel.  This is just a helper method for setConfig({logLevel: level})
    */
    BackgroundGeolocation.setLogLevel = function (value, success, failure) {
        if (arguments.length == 1) {
            return this.api.setLogLevel(value);
        }
        else {
            this.api.setLogLevel(value).then(success).catch(failure);
        }
    };
    /**
    * Fetch the entire contents of log database returned as a String
    */
    BackgroundGeolocation.getLog = function (success, failure) {
        if (!arguments.length) {
            return this.api.getLog();
        }
        else {
            this.api.getLog().then(success).catch(failure);
        }
    };
    /**
    * Destroy all contents of log database
    */
    BackgroundGeolocation.destroyLog = function (success, failure) {
        if (!arguments.length) {
            return this.api.destroyLog();
        }
        else {
            this.api.destroyLog().then(success).catch(failure);
        }
    };
    /**
    * Open deafult email client on device to email the contents of log database attached as a compressed file attachement
    */
    BackgroundGeolocation.emailLog = function (email, success, failure) {
        if (typeof (email) != 'string') {
            throw TAG + "#emailLog requires an email address as 1st argument";
        }
        if (arguments.length == 1) {
            return this.api.emailLog(email);
        }
        else {
            this.api.emailLog(email).then(success).catch(failure);
        }
    };
    /**
    * Has device OS initiated power-saving mode?
    */
    BackgroundGeolocation.isPowerSaveMode = function (success, failure) {
        if (!arguments.length) {
            return this.api.isPowerSaveMode();
        }
        else {
            this.api.isPowerSaveMode().then(success).catch(failure);
        }
    };
    /**
    * Fetch the state of this device's available motion-sensors
    */
    BackgroundGeolocation.getSensors = function (success, failure) {
        if (!arguments.length) {
            return this.api.getSensors();
        }
        else {
            this.api.getSensors().then(success).catch(failure);
        }
    };
    /**
    * Play a system sound via the plugin's Sound API
    */
    BackgroundGeolocation.playSound = function (soundId, success, failure) {
        if (arguments.length == 1) {
            return this.api.playSound(soundId);
        }
        else {
            this.api.playSound(soundId).then(success).catch(failure);
        }
    };
    BackgroundGeolocation.validate = function (config) {
        if (config.username) {
            config.url = TEST_SERVER_URL + config.username;
            config.params = {
                device: {
                    uuid: Platform.device.uuid,
                    model: Platform.device.model,
                    platform: Platform.device.os,
                    manufacturer: Platform.device.manufacturer,
                    version: Platform.device.osVersion,
                    framework: '{N}'
                }
            };
            delete config.username;
        }
        return config;
    };
    BackgroundGeolocation.LOG_LEVEL_OFF = 0;
    BackgroundGeolocation.LOG_LEVEL_ERROR = 1;
    BackgroundGeolocation.LOG_LEVEL_WARNING = 2;
    BackgroundGeolocation.LOG_LEVEL_INFO = 3;
    BackgroundGeolocation.LOG_LEVEL_DEBUG = 4;
    BackgroundGeolocation.LOG_LEVEL_VERBOSE = 5;
    BackgroundGeolocation.DESIRED_ACCURACY_NAVIGATION = -2;
    BackgroundGeolocation.DESIRED_ACCURACY_HIGH = -1;
    BackgroundGeolocation.DESIRED_ACCURACY_MEDIUM = 10;
    BackgroundGeolocation.DESIRED_ACCURACY_LOW = 100;
    BackgroundGeolocation.DESIRED_ACCURACY_VERY_LOW = 1000;
    BackgroundGeolocation.DESIRED_ACCURACY_LOWEST = 3000;
    BackgroundGeolocation.AUTHORIZATION_STATUS_NOT_DETERMINED = 0;
    BackgroundGeolocation.AUTHORIZATION_STATUS_RESTRICTED = 1;
    BackgroundGeolocation.AUTHORIZATION_STATUS_DENIED = 2;
    BackgroundGeolocation.AUTHORIZATION_STATUS_ALWAYS = 3;
    BackgroundGeolocation.AUTHORIZATION_STATUS_WHEN_IN_USE = 4;
    BackgroundGeolocation.NOTIFICATION_PRIORITY_DEFAULT = 0;
    BackgroundGeolocation.NOTIFICATION_PRIORITY_HIGH = 1;
    BackgroundGeolocation.NOTIFICATION_PRIORITY_LOW = -1;
    BackgroundGeolocation.NOTIFICATION_PRIORITY_MAX = 2;
    BackgroundGeolocation.NOTIFICATION_PRIORITY_MIN = -2;
    BackgroundGeolocation.EVENTS = [
        'heartbeat',
        'http',
        'location',
        'error',
        'motionchange',
        'geofence',
        'schedule',
        'activitychange',
        'providerchange',
        'geofenceschange',
        'watchposition',
        'powersavechange',
        'connectivitychange',
        'enabledchange'
    ];
    BackgroundGeolocation.listeners = {
        location: [],
        http: [],
        motionchange: [],
        error: [],
        heartbeat: [],
        schedule: [],
        activitychange: [],
        providerchange: [],
        geofence: [],
        geofenceschange: [],
        powersavechange: [],
        connectivitychange: [],
        enabledchange: []
    };
    return BackgroundGeolocation;
}());
exports.BackgroundGeolocation = BackgroundGeolocation;
